@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -32,7 +32,8 @@ import java.io.*;
 public class ArgumentWordReader extends WordReader
 {
     private final String[] arguments;
-    private int      index = 0;
+
+    private int index = 0;
 
 
 //    /**
@@ -86,7 +87,7 @@ public class ArgumentWordReader extends WordReader
             {
                 while (true)
                 {
-                    String word = reader.nextWord();
+                    String word = reader.nextWord(false);
                     if (word == null)
                         System.exit(-1);
 
