@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -29,23 +29,17 @@ import java.net.URL;
  *
  * @author Eric Lafortune
  */
-public class FileWordReader extends WordReader
+public class FileWordReader extends LineWordReader
 {
-    private final String           name;
-    private LineNumberReader reader;
-
-
     /**
      * Creates a new FileWordReader for the given file.
      */
     public FileWordReader(File file) throws IOException
     {
-        super(file.getParentFile());
-
-        this.name   = file.getPath();
-        this.reader = new LineNumberReader(
-                      new BufferedReader(
-                      new FileReader(file)));
+        super(new LineNumberReader(new BufferedReader(new FileReader(file))),
+              "file '" + file.getPath() + "'",
+              file.getParentFile()
+        );
     }
 
 
@@ -54,36 +48,8 @@ public class FileWordReader extends WordReader
      */
     public FileWordReader(URL url) throws IOException
     {
-        super(null);
-
-        this.name   = url.toString();
-        this.reader = new LineNumberReader(
-                       new BufferedReader(
-                       new InputStreamReader(url.openStream())));
-    }
-
-
-    // Implementations for WordReader.
-
-    protected String nextLine() throws IOException
-    {
-        return reader.readLine();
-    }
-
-
-    protected String lineLocationDescription()
-    {
-        return "line " + reader.getLineNumber() + " of file '" + name + "'";
-    }
-
-
-    public void close() throws IOException
-    {
-        super.close();
-
-        if (reader != null)
-        {
-            reader.close();
-        }
+        super(new LineNumberReader(new BufferedReader(new InputStreamReader(url.openStream()))),
+              "file '" + url.toString() + "'",
+              null);
     }
 }
