@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -60,6 +60,11 @@ public class Initializer
     {
         int originalLibraryClassPoolSize = libraryClassPool.size();
 
+        // Perform a basic check on the keep options in the configuration.
+        WarningPrinter keepClassMemberNotePrinter = new WarningPrinter(System.out, configuration.note);
+
+        new KeepClassMemberChecker(keepClassMemberNotePrinter).checkClassSpecifications(configuration.keep);
+
         // Construct a reduced library class pool with only those library
         // classes whose hierarchies are referenced by the program classes.
         // We can't do this if we later have to come up with the obfuscated
@@ -248,7 +253,7 @@ public class Initializer
         {
             System.out.println("Note: there were " + dynamicClassReferenceNoteCount +
                                " unresolved dynamic references to classes or interfaces.");
-            System.err.println("      You should check if you need to specify additional program jars.");
+            System.out.println("      You should check if you need to specify additional program jars.");
         }
 
         int classForNameNoteCount = classForNameNotePrinter.getWarningCount();
