@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -90,6 +90,16 @@ public interface Clazz extends VisitorAccepter
      */
     public String getType(int constantIndex);
 
+    /**
+     * Returns the name of the RefConstant at the specified index.
+     */
+    public String getRefName(int constantIndex);
+
+    /**
+     * Returns the type of the RefConstant at the specified index.
+     */
+    public String getRefType(int constantIndex);
+
 
     // Methods pertaining to related classes.
 
@@ -115,6 +125,13 @@ public interface Clazz extends VisitorAccepter
      */
     public boolean extends_(Clazz clazz);
 
+    /**
+     * Returns whether this class extends the specified class.
+     * A class is always considered to extend itself.
+     * Interfaces are considered to only extend the root Object class.
+     */
+    public boolean extends_(String className);
+
     /**
      * Returns whether this class implements the given class.
      * A class is always considered to implement itself.
@@ -122,6 +139,13 @@ public interface Clazz extends VisitorAccepter
      */
     public boolean extendsOrImplements(Clazz clazz);
 
+    /**
+     * Returns whether this class implements the specified class.
+     * A class is always considered to implement itself.
+     * Interfaces are considered to implement all their superinterfaces.
+     */
+    public boolean extendsOrImplements(String className);
+
 
     // Methods for getting specific class members.
 
@@ -229,4 +253,9 @@ public interface Clazz extends VisitorAccepter
      * Lets the given attribute info visitor visit all attributes of this class.
      */
     public void attributesAccept(AttributeVisitor attributeVisitor);
+
+    /**
+     * Lets the given attribute info visitor visit the specified attribute.
+     */
+    public void attributeAccept(String name, AttributeVisitor attributeVisitor);
 }
