@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -28,7 +28,7 @@ import proguard.classfile.visitor.ClassVisitor;
  *
  * @author Eric Lafortune
  */
-public class LocalVariableTypeInfo
+public class LocalVariableTypeInfo implements Comparable
 {
     public int u2startPC;
     public int u2length;
@@ -88,4 +88,20 @@ public class LocalVariableTypeInfo
             }
         }
     }
+
+
+    // Implementations for Comparable.
+
+    public int compareTo(Object object)
+    {
+        LocalVariableTypeInfo other = (LocalVariableTypeInfo)object;
+
+        return
+            this.u2startPC         < other.u2startPC         ? -1 : this.u2startPC         > other.u2startPC         ? 1 :
+            this.u2length          < other.u2length          ? -1 : this.u2length          > other.u2length          ? 1 :
+            this.u2index           < other.u2index           ? -1 : this.u2index           > other.u2index           ? 1 :
+            this.u2signatureIndex  < other.u2signatureIndex  ? -1 : this.u2signatureIndex  > other.u2signatureIndex  ? 1 :
+            this.u2nameIndex       < other.u2nameIndex       ? -1 : this.u2nameIndex       > other.u2nameIndex       ? 1 :
+                                                                                                                       0;
+    }
 }
