@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -27,6 +27,8 @@ import proguard.classfile.instruction.*;
 import proguard.classfile.instruction.visitor.InstructionVisitor;
 import proguard.classfile.util.SimplifiedVisitor;
 
+import java.util.Arrays;
+
 /**
  * This AttributeVisitor computes the stack sizes at all instruction offsets
  * of the code attributes that it visits.
@@ -140,10 +142,7 @@ implements   AttributeVisitor,
         }
         else
         {
-            for (int index = 0; index < codeLength; index++)
-            {
-                evaluated[index] = false;
-            }
+            Arrays.fill(evaluated, 0, codeLength, false);
         }
 
         // The initial stack is always empty.
@@ -276,7 +275,7 @@ implements   AttributeVisitor,
 
     /**
      * Evaluates a block of instructions that hasn't been handled before,
-     * starting at the given offset and ending a branch instruction, a return
+     * starting at the given offset and ending at a branch instruction, a return
      * instruction, or a throw instruction. Branch instructions are handled
      * recursively.
      */
