@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -29,6 +29,8 @@ import proguard.classfile.instruction.*;
 import proguard.classfile.instruction.visitor.InstructionVisitor;
 import proguard.classfile.util.SimplifiedVisitor;
 
+import java.util.Arrays;
+
 /**
  * This AttributeVisitor accumulates specified changes to code, and then applies
  * these accumulated changes to the code attributes that it visits.
@@ -104,13 +106,10 @@ implements   AttributeVisitor,
         }
         else
         {
-            for (int index = 0; index < codeLength; index++)
-            {
-                preInsertions[index]  = null;
-                replacements[index]   = null;
-                postInsertions[index] = null;
-                deleted[index]        = false;
-            }
+            Arrays.fill(preInsertions,  0, codeLength, null);
+            Arrays.fill(replacements,   0, codeLength, null);
+            Arrays.fill(postInsertions, 0, codeLength, null);
+            Arrays.fill(deleted,        0, codeLength, false);
         }
 
         modified = false;
@@ -141,6 +140,28 @@ implements   AttributeVisitor,
     }
 
 
+    /**
+     * Remembers to place the given instructions right before the instruction
+     * at the given offset.
+     * @param instructionOffset the offset of the instruction.
+     * @param instructions      the new instructions.
+     */
+    public void insertBeforeInstruction(int instructionOffset, Instruction[] instructions)
+    {
+        if (instructionOffset < 0 ||
+            instructionOffset >= codeLength)
+        {
+            throw new IllegalArgumentException("Invalid instruction offset ["+instructionOffset+"] in code with length ["+codeLength+"]");
+        }
+
+        preInsertions[instructionOffset] = new CompositeInstruction(instructions);
+
+        modified = true;
+        simple   = false;
+
+    }
+
+
     /**
      * Remembers to replace the instruction at the given offset by the given
      * instruction.
@@ -161,6 +182,26 @@ implements   AttributeVisitor,
     }
 
 
+    /**
+     * Remembers to replace the instruction at the given offset by the given
+     * instructions.
+     * @param instructionOffset the offset of the instruction to be replaced.
+     * @param instructions      the new instructions.
+     */
+    public void replaceInstruction(int instructionOffset, Instruction[] instructions)
+    {
+        if (instructionOffset < 0 ||
+            instructionOffset >= codeLength)
+        {
+            throw new IllegalArgumentException("Invalid instruction offset ["+instructionOffset+"] in code with length ["+codeLength+"]");
+        }
+
+        replacements[instructionOffset] = new CompositeInstruction(instructions);
+
+        modified = true;
+    }
+
+
     /**
      * Remembers to place the given instruction right after the instruction
      * at the given offset.
@@ -182,6 +223,27 @@ implements   AttributeVisitor,
     }
 
 
+    /**
+     * Remembers to place the given instructions right after the instruction
+     * at the given offset.
+     * @param instructionOffset the offset of the instruction.
+     * @param instructions      the new instructions.
+     */
+    public void insertAfterInstruction(int instructionOffset, Instruction[] instructions)
+    {
+        if (instructionOffset < 0 ||
+            instructionOffset >= codeLength)
+        {
+            throw new IllegalArgumentException("Invalid instruction offset ["+instructionOffset+"] in code with length ["+codeLength+"]");
+        }
+
+        postInsertions[instructionOffset] = new CompositeInstruction(instructions);
+
+        modified = true;
+        simple   = false;
+    }
+
+
     /**
      * Remembers to delete the instruction at the given offset.
      * @param instructionOffset the offset of the instruction to be deleted.
@@ -264,52 +326,51 @@ implements   AttributeVisitor,
     {
         if (DEBUG)
         {
-            System.out.println("CodeAttributeEditor: ["+clazz.getName()+"."+method.getName(clazz)+"]");
+            System.out.println("CodeAttributeEditor: "+clazz.getName()+"."+method.getName(clazz)+method.getDescriptor(clazz));
         }
 
-        // Avoid doing any work if nothing is changing anyway.
-        if (!modified)
+        // Do we have to update the code?
+        if (modified)
         {
-            return;
-        }
+            // Can we perform a faster simple replacement of instructions?
+            if (canPerformSimpleReplacements(codeAttribute))
+            {
+                if (DEBUG)
+                {
+                    System.out.println("  Simple editing");
+                }
 
-        // Check if we can perform a faster simple replacement of instructions.
-        if (canPerformSimpleReplacements(codeAttribute))
-        {
-            // Simply overwrite the instructions.
-            performSimpleReplacements(codeAttribute);
+                // Simply overwrite the instructions.
+                performSimpleReplacements(codeAttribute);
+            }
+            else
+            {
+                if (DEBUG)
+                {
+                    System.out.println("  Full editing");
+                }
 
-            // Update the maximum stack size and local variable frame size.
-            updateFrameSizes(clazz, method, codeAttribute);
-        }
-        else
-        {
-            // Move and remap the instructions.
-            codeAttribute.u4codeLength =
-                updateInstructions(clazz, method, codeAttribute);
+                // Move and remap the instructions.
+                codeAttribute.u4codeLength =
+                    updateInstructions(clazz, method, codeAttribute);
 
-            // Remap the exception table.
-            codeAttribute.exceptionsAccept(clazz, method, this);
+                // Remap the exception table.
+                codeAttribute.exceptionsAccept(clazz, method, this);
 
-            // Remove exceptions with empty code blocks.
-            codeAttribute.u2exceptionTableLength =
-                removeEmptyExceptions(codeAttribute.exceptionTable,
-                                      codeAttribute.u2exceptionTableLength);
+                // Remove exceptions with empty code blocks.
+                codeAttribute.u2exceptionTableLength =
+                    removeEmptyExceptions(codeAttribute.exceptionTable,
+                                          codeAttribute.u2exceptionTableLength);
 
-            // Update the maximum stack size and local variable frame size.
-            updateFrameSizes(clazz, method, codeAttribute);
-
-            // Remap the line number table and the local variable table.
-            codeAttribute.attributesAccept(clazz, method, this);
+                // Remap the line number table and the local variable tables.
+                codeAttribute.attributesAccept(clazz, method, this);
+            }
 
             // Make sure instructions are widened if necessary.
             instructionWriter.visitCodeAttribute(clazz, method, codeAttribute);
         }
-    }
 
-
-    private void updateFrameSizes(Clazz clazz, Method method, CodeAttribute codeAttribute)
-    {
+        // Update the maximum stack size and local variable frame size.
         if (updateFrameSizes)
         {
             stackSizeUpdater.visitCodeAttribute(clazz, method, codeAttribute);
@@ -351,12 +412,6 @@ implements   AttributeVisitor,
     {
         // Remap all local variable table entries.
         localVariableTableAttribute.localVariablesAccept(clazz, method, codeAttribute, this);
-
-        // Remove local variables with empty code blocks.
-        localVariableTableAttribute.u2localVariableTableLength =
-            removeEmptyLocalVariables(localVariableTableAttribute.localVariableTable,
-                                      localVariableTableAttribute.u2localVariableTableLength,
-                                      codeAttribute.u2maxLocals);
     }
 
 
@@ -364,12 +419,6 @@ implements   AttributeVisitor,
     {
         // Remap all local variable table entries.
         localVariableTypeTableAttribute.localVariablesAccept(clazz, method, codeAttribute, this);
-
-        // Remove local variables with empty code blocks.
-        localVariableTypeTableAttribute.u2localVariableTypeTableLength =
-            removeEmptyLocalVariableTypes(localVariableTypeTableAttribute.localVariableTypeTable,
-                                          localVariableTypeTableAttribute.u2localVariableTypeTableLength,
-                                          codeAttribute.u2maxLocals);
     }
 
 
@@ -610,15 +659,13 @@ implements   AttributeVisitor,
         Instruction preInstruction = preInsertions[oldOffset];
         if (preInstruction != null)
         {
-            // Remap the instruction.
-            preInstruction.accept(clazz, method, codeAttribute, oldOffset, this);
-
             if (DEBUG)
             {
                 System.out.println("  Pre-inserted  "+preInstruction.toString(newOffset));
             }
 
-            newOffset += preInstruction.length(newOffset);
+            // Remap the instruction.
+            preInstruction.accept(clazz, method, codeAttribute, oldOffset, this);
         }
 
         // Remap and insert the replacement instruction, or the current
@@ -626,42 +673,35 @@ implements   AttributeVisitor,
         Instruction replacementInstruction = replacements[oldOffset];
         if (replacementInstruction != null)
         {
-            // Remap the instruction.
-            replacementInstruction.accept(clazz, method, codeAttribute, oldOffset, this);
-
             if (DEBUG)
             {
                 System.out.println("  Replaced      "+replacementInstruction.toString(newOffset));
             }
-
-            newOffset += replacementInstruction.length(newOffset);
+            // Remap the instruction.
+            replacementInstruction.accept(clazz, method, codeAttribute, oldOffset, this);
         }
         else if (!deleted[oldOffset])
         {
-            // Remap the instruction.
-            instruction.accept(clazz, method, codeAttribute, oldOffset, this);
-
             if (DEBUG)
             {
                 System.out.println("  Copied        "+instruction.toString(newOffset));
             }
 
-            newOffset += instruction.length(newOffset);
+            // Remap the instruction.
+            instruction.accept(clazz, method, codeAttribute, oldOffset, this);
         }
 
         // Remap and insert the post-inserted instruction, if any.
         Instruction postInstruction = postInsertions[oldOffset];
         if (postInstruction != null)
         {
-            // Remap the instruction.
-            postInstruction.accept(clazz, method, codeAttribute, oldOffset, this);
-
             if (DEBUG)
             {
                 System.out.println("  Post-inserted "+postInstruction.toString(newOffset));
             }
 
-            newOffset += postInstruction.length(newOffset);
+            // Remap the instruction.
+            postInstruction.accept(clazz, method, codeAttribute, oldOffset, this);
         }
     }
 
@@ -676,6 +716,8 @@ implements   AttributeVisitor,
                                                  codeAttribute,
                                                  newOffset,
                                                  simpleInstruction);
+
+        newOffset += simpleInstruction.length(newOffset);
     }
 
 
@@ -687,6 +729,8 @@ implements   AttributeVisitor,
                                                    codeAttribute,
                                                    newOffset,
                                                    constantInstruction);
+
+        newOffset += constantInstruction.length(newOffset);
     }
 
 
@@ -698,6 +742,8 @@ implements   AttributeVisitor,
                                                    codeAttribute,
                                                    newOffset,
                                                    variableInstruction);
+
+        newOffset += variableInstruction.length(newOffset);
     }
 
 
@@ -713,6 +759,8 @@ implements   AttributeVisitor,
                                                  codeAttribute,
                                                  newOffset,
                                                  branchInstruction);
+
+        newOffset += branchInstruction.length(newOffset);
     }
 
 
@@ -732,6 +780,8 @@ implements   AttributeVisitor,
                                                       codeAttribute,
                                                       newOffset,
                                                       tableSwitchInstruction);
+
+        newOffset += tableSwitchInstruction.length(newOffset);
     }
 
 
@@ -751,6 +801,8 @@ implements   AttributeVisitor,
                                                        codeAttribute,
                                                        newOffset,
                                                        lookUpSwitchInstruction);
+
+        newOffset += lookUpSwitchInstruction.length(newOffset);
     }
 
 
@@ -845,10 +897,12 @@ implements   AttributeVisitor,
     public void visitLocalVariableInfo(Clazz clazz, Method method, CodeAttribute codeAttribute, LocalVariableInfo localVariableInfo)
     {
         // Remap the code offset and length.
-        // TODO: The local variable frame might not be strictly preserved.
-        localVariableInfo.u2length  = remapBranchOffset(localVariableInfo.u2startPC,
-                                                        localVariableInfo.u2length);
-        localVariableInfo.u2startPC = remapInstructionOffset(localVariableInfo.u2startPC);
+        int newStartPC = remapInstructionOffset(localVariableInfo.u2startPC);
+        int newEndPC   = remapInstructionOffset(localVariableInfo.u2startPC +
+                                                localVariableInfo.u2length);
+
+        localVariableInfo.u2length  = newEndPC - newStartPC;
+        localVariableInfo.u2startPC = newStartPC;
     }
 
 
@@ -857,10 +911,12 @@ implements   AttributeVisitor,
     public void visitLocalVariableTypeInfo(Clazz clazz, Method method, CodeAttribute codeAttribute, LocalVariableTypeInfo localVariableTypeInfo)
     {
         // Remap the code offset and length.
-        // TODO: The local variable frame might not be strictly preserved.
-        localVariableTypeInfo.u2length  = remapBranchOffset(localVariableTypeInfo.u2startPC,
-                                                            localVariableTypeInfo.u2length);
-        localVariableTypeInfo.u2startPC = remapInstructionOffset(localVariableTypeInfo.u2startPC);
+        int newStartPC = remapInstructionOffset(localVariableTypeInfo.u2startPC);
+        int newEndPC   = remapInstructionOffset(localVariableTypeInfo.u2startPC +
+                                                localVariableTypeInfo.u2length);
+
+        localVariableTypeInfo.u2length  = newEndPC - newStartPC;
+        localVariableTypeInfo.u2startPC = newStartPC;
     }
 
 
@@ -950,50 +1006,99 @@ implements   AttributeVisitor,
     }
 
 
-    /**
-     * Returns the given list of local variables, without the ones that have empty
-     * code blocks or that exceed the actual number of local variables.
-     */
-    private int removeEmptyLocalVariables(LocalVariableInfo[] localVariableInfos,
-                                          int                 localVariableInfoCount,
-                                          int                 maxLocals)
+    private class CompositeInstruction
+    extends       Instruction
     {
-        // Overwrite all empty local variable entries.
-        int newIndex = 0;
-        for (int index = 0; index < localVariableInfoCount; index++)
+        private Instruction[] instructions;
+
+
+        private CompositeInstruction(Instruction[] instructions)
         {
-            LocalVariableInfo localVariableInfo = localVariableInfos[index];
-            if (localVariableInfo.u2length > 0 &&
-                localVariableInfo.u2index < maxLocals)
+            this.instructions = instructions;
+        }
+
+
+        // Implementations for Instruction.
+
+        public Instruction shrink()
+        {
+            for (int index = 0; index < instructions.length; index++)
             {
-                localVariableInfos[newIndex++] = localVariableInfo;
+                instructions[index] = instructions[index].shrink();
+            }
+
+            return this;
+        }
+
+
+        public void write(byte[] code, int offset)
+        {
+            for (int index = 0; index < instructions.length; index++)
+            {
+                Instruction instruction = instructions[index];
+
+                instruction.write(code, offset);
+
+                offset += instruction.length(offset);
             }
         }
 
-        return newIndex;
-    }
 
-
-    /**
-     * Returns the given list of local variable types, without the ones that
-     * have empty code blocks or that exceed the actual number of local variables.
-     */
-    private int removeEmptyLocalVariableTypes(LocalVariableTypeInfo[] localVariableTypeInfos,
-                                              int                     localVariableTypeInfoCount,
-                                              int                     maxLocals)
-    {
-        // Overwrite all empty local variable type entries.
-        int newIndex = 0;
-        for (int index = 0; index < localVariableTypeInfoCount; index++)
+        protected void readInfo(byte[] code, int offset)
         {
-            LocalVariableTypeInfo localVariableTypeInfo = localVariableTypeInfos[index];
-            if (localVariableTypeInfo.u2length > 0 &&
-                localVariableTypeInfo.u2index < maxLocals)
+            throw new UnsupportedOperationException("Can't read composite instruction");
+        }
+
+
+        protected void writeInfo(byte[] code, int offset)
+        {
+            throw new UnsupportedOperationException("Can't write composite instruction");
+        }
+
+
+        public int length(int offset)
+        {
+            int newOffset = offset;
+
+            for (int index = 0; index < instructions.length; index++)
             {
-                localVariableTypeInfos[newIndex++] = localVariableTypeInfo;
+                newOffset += instructions[index].length(newOffset);
+            }
+
+            return newOffset - offset;
+        }
+
+
+        public void accept(Clazz clazz, Method method, CodeAttribute codeAttribute, int offset, InstructionVisitor instructionVisitor)
+        {
+            if (instructionVisitor != CodeAttributeEditor.this)
+            {
+                throw new UnsupportedOperationException("Unexpected visitor ["+instructionVisitor+"]");
+            }
+
+            for (int index = 0; index < instructions.length; index++)
+            {
+                Instruction instruction = instructions[index];
+
+                instruction.accept(clazz, method, codeAttribute, offset, CodeAttributeEditor.this);
+
+                offset += instruction.length(offset);
             }
         }
 
-        return newIndex;
+
+        // Implementations for Object.
+
+        public String toString()
+        {
+            StringBuffer stringBuffer = new StringBuffer();
+
+            for (int index = 0; index < instructions.length; index++)
+            {
+                stringBuffer.append(instructions[index].toString()).append("; ");
+            }
+
+            return stringBuffer.toString();
+        }
     }
 }
