@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -47,9 +47,10 @@ implements   ClassVisitor,
              ConstantVisitor,
              MemberVisitor,
              AttributeVisitor,
-             InstructionVisitor,
+             BootstrapMethodInfoVisitor,
              InnerClassesInfoVisitor,
              ExceptionInfoVisitor,
+             InstructionVisitor,
              StackMapFrameVisitor,
              VerificationTypeVisitor,
              LocalVariableInfoVisitor,
@@ -57,7 +58,7 @@ implements   ClassVisitor,
              AnnotationVisitor,
              ElementValueVisitor
 {
-    private final CodeAttributeEditor codeAttributeEditor = new CodeAttributeEditor();
+    private final CodeAttributeEditor codeAttributeEditor = new CodeAttributeEditor(false);
 
     private int[] constantIndexMap;
 
@@ -100,70 +101,27 @@ implements   ClassVisitor,
 
     // Implementations for ConstantVisitor.
 
-    public void visitClassConstant(Clazz clazz, ClassConstant classConstant)
-    {
-        classConstant.u2nameIndex =
-            remapConstantIndex(classConstant.u2nameIndex);
-    }
-
-
-    public void visitDoubleConstant(Clazz clazz, DoubleConstant doubleConstant)
-    {
-        // Nothing to do.
-    }
-
-
-    public void visitFieldrefConstant(Clazz clazz, FieldrefConstant fieldrefConstant)
-    {
-        fieldrefConstant.u2classIndex =
-            remapConstantIndex(fieldrefConstant.u2classIndex);
-        fieldrefConstant.u2nameAndTypeIndex =
-            remapConstantIndex(fieldrefConstant.u2nameAndTypeIndex);
-    }
-
-
-    public void visitFloatConstant(Clazz clazz, FloatConstant floatConstant)
-    {
-        // Nothing to do.
-    }
-
-
     public void visitIntegerConstant(Clazz clazz, IntegerConstant integerConstant)
     {
         // Nothing to do.
     }
 
 
-    public void visitInterfaceMethodrefConstant(Clazz clazz, InterfaceMethodrefConstant interfaceMethodrefConstant)
-    {
-        interfaceMethodrefConstant.u2classIndex =
-            remapConstantIndex(interfaceMethodrefConstant.u2classIndex);
-        interfaceMethodrefConstant.u2nameAndTypeIndex =
-            remapConstantIndex(interfaceMethodrefConstant.u2nameAndTypeIndex);
-    }
-
-
     public void visitLongConstant(Clazz clazz, LongConstant longConstant)
     {
         // Nothing to do.
     }
 
 
-    public void visitMethodrefConstant(Clazz clazz, MethodrefConstant methodrefConstant)
+    public void visitFloatConstant(Clazz clazz, FloatConstant floatConstant)
     {
-        methodrefConstant.u2classIndex =
-            remapConstantIndex(methodrefConstant.u2classIndex);
-        methodrefConstant.u2nameAndTypeIndex =
-            remapConstantIndex(methodrefConstant.u2nameAndTypeIndex);
+        // Nothing to do.
     }
 
 
-    public void visitNameAndTypeConstant(Clazz clazz, NameAndTypeConstant nameAndTypeConstant)
+    public void visitDoubleConstant(Clazz clazz, DoubleConstant doubleConstant)
     {
-        nameAndTypeConstant.u2nameIndex =
-            remapConstantIndex(nameAndTypeConstant.u2nameIndex);
-        nameAndTypeConstant.u2descriptorIndex =
-            remapConstantIndex(nameAndTypeConstant.u2descriptorIndex);
+        // Nothing to do.
     }
 
 
@@ -180,6 +138,70 @@ implements   ClassVisitor,
     }
 
 
+    public void visitInvokeDynamicConstant(Clazz clazz, InvokeDynamicConstant invokeDynamicConstant)
+    {
+        invokeDynamicConstant.u2nameAndTypeIndex =
+            remapConstantIndex(invokeDynamicConstant.u2nameAndTypeIndex);
+    }
+
+
+    public void visitMethodHandleConstant(Clazz clazz, MethodHandleConstant methodHandleConstant)
+    {
+        methodHandleConstant.u2referenceIndex =
+            remapConstantIndex(methodHandleConstant.u2referenceIndex);
+    }
+
+
+    public void visitFieldrefConstant(Clazz clazz, FieldrefConstant fieldrefConstant)
+    {
+        fieldrefConstant.u2classIndex =
+            remapConstantIndex(fieldrefConstant.u2classIndex);
+        fieldrefConstant.u2nameAndTypeIndex =
+            remapConstantIndex(fieldrefConstant.u2nameAndTypeIndex);
+    }
+
+
+    public void visitInterfaceMethodrefConstant(Clazz clazz, InterfaceMethodrefConstant interfaceMethodrefConstant)
+    {
+        interfaceMethodrefConstant.u2classIndex =
+            remapConstantIndex(interfaceMethodrefConstant.u2classIndex);
+        interfaceMethodrefConstant.u2nameAndTypeIndex =
+            remapConstantIndex(interfaceMethodrefConstant.u2nameAndTypeIndex);
+    }
+
+
+    public void visitMethodrefConstant(Clazz clazz, MethodrefConstant methodrefConstant)
+    {
+        methodrefConstant.u2classIndex =
+            remapConstantIndex(methodrefConstant.u2classIndex);
+        methodrefConstant.u2nameAndTypeIndex =
+            remapConstantIndex(methodrefConstant.u2nameAndTypeIndex);
+    }
+
+
+    public void visitClassConstant(Clazz clazz, ClassConstant classConstant)
+    {
+        classConstant.u2nameIndex =
+            remapConstantIndex(classConstant.u2nameIndex);
+    }
+
+
+    public void visitMethodTypeConstant(Clazz clazz, MethodTypeConstant methodTypeConstant)
+    {
+        methodTypeConstant.u2descriptorIndex =
+            remapConstantIndex(methodTypeConstant.u2descriptorIndex);
+    }
+
+
+    public void visitNameAndTypeConstant(Clazz clazz, NameAndTypeConstant nameAndTypeConstant)
+    {
+        nameAndTypeConstant.u2nameIndex =
+            remapConstantIndex(nameAndTypeConstant.u2nameIndex);
+        nameAndTypeConstant.u2descriptorIndex =
+            remapConstantIndex(nameAndTypeConstant.u2descriptorIndex);
+    }
+
+
     // Implementations for MemberVisitor.
 
     public void visitProgramField(ProgramClass programClass, ProgramField programField)
@@ -230,6 +252,16 @@ implements   ClassVisitor,
     }
 
 
+    public void visitBootstrapMethodsAttribute(Clazz clazz, BootstrapMethodsAttribute bootstrapMethodsAttribute)
+    {
+        bootstrapMethodsAttribute.u2attributeNameIndex =
+            remapConstantIndex(bootstrapMethodsAttribute.u2attributeNameIndex);
+
+        // Remap the constant pool references of the bootstrap method entries.
+        bootstrapMethodsAttribute.bootstrapMethodEntriesAccept(clazz, this);
+    }
+
+
     public void visitSourceFileAttribute(Clazz clazz, SourceFileAttribute sourceFileAttribute)
     {
         sourceFileAttribute.u2attributeNameIndex =
@@ -410,6 +442,19 @@ implements   ClassVisitor,
     }
 
 
+    // Implementations for BootstrapMethodInfoVisitor.
+
+    public void visitBootstrapMethodInfo(Clazz clazz, BootstrapMethodInfo bootstrapMethodInfo)
+    {
+        bootstrapMethodInfo.u2methodHandleIndex =
+            remapConstantIndex(bootstrapMethodInfo.u2methodHandleIndex);
+
+        // Remap the constant pool references of the bootstrap methods..
+        remapConstantIndexArray(bootstrapMethodInfo.u2methodArguments,
+                                bootstrapMethodInfo.u2methodArgumentCount);
+    }
+
+
     // Implementations for InnerClassesInfoVisitor.
 
     public void visitInnerClassesInfo(Clazz clazz, InnerClassesInfo innerClassesInfo)
