@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -29,7 +29,8 @@ import proguard.classfile.util.*;
 
 /**
  * This AttributeVisitor computes and updates the maximum local variable frame
- * size of the code attributes that it visits.
+ * size of the code attributes that it visits. It also cleans up the local
+ * variable tables.
  *
  * @author Eric Lafortune
  */
@@ -45,6 +46,9 @@ implements   AttributeVisitor,
     //*/
 
 
+    private VariableCleaner variableCleaner = new VariableCleaner();
+
+
     // Implementations for AttributeVisitor.
 
     public void visitAnyAttribute(Clazz clazz, Attribute attribute) {}
@@ -69,6 +73,9 @@ implements   AttributeVisitor,
 
         // Go over all instructions.
         codeAttribute.instructionsAccept(clazz, method, this);
+
+        // Remove the unused variables of the attributes.
+        variableCleaner.visitCodeAttribute(clazz, method, codeAttribute);
     }
 
 
@@ -91,7 +98,7 @@ implements   AttributeVisitor,
 
             if (DEBUG)
             {
-                System.out.println("Max locals: "+codeAttribute.u2maxLocals+" <- "+variableInstruction.toString(offset));
+                System.out.println("  Max locals: "+codeAttribute.u2maxLocals+" <- "+variableInstruction.toString(offset));
             }
         }
     }
