@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -57,7 +57,7 @@ implements   ClassVisitor,
 
 
     /**
-     * Creates a new ProgramClassWriter for reading from the given DataOutput.
+     * Creates a new ProgramClassWriter for writing to the given DataOutput.
      */
     public ProgramClassWriter(DataOutput dataOutput)
     {
@@ -208,6 +208,20 @@ implements   ClassVisitor,
         }
 
 
+        public void visitInvokeDynamicConstant(Clazz clazz, InvokeDynamicConstant invokeDynamicConstant)
+        {
+            dataOutput.writeShort(invokeDynamicConstant.u2bootstrapMethodAttributeIndex);
+            dataOutput.writeShort(invokeDynamicConstant.u2nameAndTypeIndex);
+        }
+
+
+        public void visitMethodHandleConstant(Clazz clazz, MethodHandleConstant methodHandleConstant)
+        {
+            dataOutput.writeByte(methodHandleConstant.u1referenceKind);
+            dataOutput.writeShort(methodHandleConstant.u2referenceIndex);
+        }
+
+
         public void visitAnyRefConstant(Clazz clazz, RefConstant refConstant)
         {
             dataOutput.writeShort(refConstant.u2classIndex);
@@ -221,6 +235,12 @@ implements   ClassVisitor,
         }
 
 
+        public void visitMethodTypeConstant(Clazz clazz, MethodTypeConstant methodTypeConstant)
+        {
+            dataOutput.writeShort(methodTypeConstant.u2descriptorIndex);
+        }
+
+
         public void visitNameAndTypeConstant(Clazz clazz, NameAndTypeConstant nameAndTypeConstant)
         {
             dataOutput.writeShort(nameAndTypeConstant.u2nameIndex);
@@ -263,6 +283,7 @@ implements   ClassVisitor,
     private class AttributeBodyWriter
     extends       SimplifiedVisitor
     implements    AttributeVisitor,
+                  BootstrapMethodInfoVisitor,
                   InnerClassesInfoVisitor,
                   ExceptionInfoVisitor,
                   StackMapFrameVisitor,
@@ -282,6 +303,15 @@ implements   ClassVisitor,
         }
 
 
+        public void visitBootstrapMethodsAttribute(Clazz clazz, BootstrapMethodsAttribute bootstrapMethodsAttribute)
+        {
+            // Write the bootstrap methods.
+            dataOutput.writeShort(bootstrapMethodsAttribute.u2bootstrapMethodsCount);
+
+            bootstrapMethodsAttribute.bootstrapMethodEntriesAccept(clazz, this);
+        }
+
+
         public void visitSourceFileAttribute(Clazz clazz, SourceFileAttribute sourceFileAttribute)
         {
             dataOutput.writeShort(sourceFileAttribute.u2sourceFileIndex);
@@ -438,8 +468,7 @@ implements   ClassVisitor,
 
                 for (int index = 0; index < u2annotationsCount; index++)
                 {
-                    Annotation annotation = annotations[index];
-                    this.visitAnnotation(clazz, annotation);
+                    visitAnnotation(clazz, annotations[index]);
                 }
 
             }
@@ -453,6 +482,22 @@ implements   ClassVisitor,
         }
 
 
+        // Implementations for BootstrapMethodInfoVisitor.
+
+        public void visitBootstrapMethodInfo(Clazz clazz, BootstrapMethodInfo bootstrapMethodInfo)
+        {
+            dataOutput.writeShort(bootstrapMethodInfo.u2methodHandleIndex);
+
+            // Write the bootstrap method arguments.
+            dataOutput.writeShort(bootstrapMethodInfo.u2methodArgumentCount);
+
+            for (int index = 0; index < bootstrapMethodInfo.u2methodArgumentCount; index++)
+            {
+                dataOutput.writeShort(bootstrapMethodInfo.u2methodArguments[index]);
+            }
+        }
+
+
         // Implementations for InnerClassesInfoVisitor.
 
         public void visitInnerClassesInfo(Clazz clazz, InnerClassesInfo innerClassesInfo)
