@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -63,7 +63,8 @@ implements   ClassVisitor,
 {
     private final ClassPool      programClassPool;
     private final ClassPool      libraryClassPool;
-    private final WarningPrinter missingWarningPrinter;
+    private final WarningPrinter missingClassWarningPrinter;
+    private final WarningPrinter missingMemberWarningPrinter;
     private final WarningPrinter dependencyWarningPrinter;
 
     private final MemberFinder memberFinder = new MemberFinder();
@@ -72,17 +73,19 @@ implements   ClassVisitor,
     /**
      * Creates a new ClassReferenceInitializer that initializes the references
      * of all visited class files, optionally printing warnings if some classes
-     * can't be found or if they are in the program class pool.
+     * or class members can't be found or if they are in the program class pool.
      */
     public ClassReferenceInitializer(ClassPool      programClassPool,
                                      ClassPool      libraryClassPool,
-                                     WarningPrinter missingWarningPrinter,
+                                     WarningPrinter missingClassWarningPrinter,
+                                     WarningPrinter missingMemberWarningPrinter,
                                      WarningPrinter dependencyWarningPrinter)
     {
-        this.programClassPool         = programClassPool;
-        this.libraryClassPool         = libraryClassPool;
-        this.missingWarningPrinter    = missingWarningPrinter;
-        this.dependencyWarningPrinter = dependencyWarningPrinter;
+        this.programClassPool            = programClassPool;
+        this.libraryClassPool            = libraryClassPool;
+        this.missingClassWarningPrinter  = missingClassWarningPrinter;
+        this.missingMemberWarningPrinter = missingMemberWarningPrinter;
+        this.dependencyWarningPrinter    = dependencyWarningPrinter;
     }
 
 
@@ -163,17 +166,30 @@ implements   ClassVisitor,
     }
 
 
+    public void visitMethodHandleConstant(Clazz clazz, MethodHandleConstant methodHandleConstant)
+    {
+        // Fill out the MethodHandle class.
+        methodHandleConstant.javaLangInvokeMethodHandleClass =
+            findClass(clazz.getName(), ClassConstants.INTERNAL_NAME_JAVA_LANG_INVOKE_METHOD_HANDLE);
+    }
+
+
     public void visitAnyRefConstant(Clazz clazz, RefConstant refConstant)
     {
         String className = refConstant.getClassName(clazz);
 
+        // Methods for array types should be found in the Object class.
+        if (ClassUtil.isInternalArrayType(className))
+        {
+            className = ClassConstants.INTERNAL_NAME_JAVA_LANG_OBJECT;
+        }
+
         // See if we can find the referenced class.
         // Unresolved references are assumed to refer to library classes
         // that will not change anyway.
         Clazz referencedClass = findClass(clazz.getName(), className);
 
-        if (referencedClass != null &&
-            !ClassUtil.isInternalArrayType(className))
+        if (referencedClass != null)
         {
             String name = refConstant.getName(clazz);
             String type = refConstant.getType(clazz);
@@ -189,18 +205,19 @@ implements   ClassVisitor,
                                                                    isFieldRef);
             refConstant.referencedClass  = memberFinder.correspondingClass();
 
-            if (refConstant.referencedMember == null &&
-                missingWarningPrinter != null)
+            if (refConstant.referencedMember == null)
             {
-                // We've haven't found the class member anywhere in the hierarchy.
-                missingWarningPrinter.print("Warning: " +
-                                     ClassUtil.externalClassName(clazz.getName()) +
-                                     ": can't find referenced " +
-                                     (isFieldRef ?
-                                         "field '"  + ClassUtil.externalFullFieldDescription(0, name, type) :
-                                         "method '" + ClassUtil.externalFullMethodDescription(className, 0, name, type)) +
-                                     "' in class " +
-                                     ClassUtil.externalClassName(className));
+                // We haven't found the class member anywhere in the hierarchy.
+                missingMemberWarningPrinter.print(clazz.getName(),
+                                                  className,
+                                                  "Warning: " +
+                                                  ClassUtil.externalClassName(clazz.getName()) +
+                                                  ": can't find referenced " +
+                                                  (isFieldRef ?
+                                                      "field '"  + ClassUtil.externalFullFieldDescription(0, name, type) :
+                                                      "method '" + ClassUtil.externalFullMethodDescription(className, 0, name, type)) +
+                                                  "' in class " +
+                                                  ClassUtil.externalClassName(className));
             }
         }
     }
@@ -212,7 +229,7 @@ implements   ClassVisitor,
 
         // Fill out the referenced class.
         classConstant.referencedClass =
-            findClass(className, classConstant.getName(clazz));
+            findClass(className, ClassUtil.internalClassNameFromClassType(classConstant.getName(clazz)));
 
         // Fill out the Class class.
         classConstant.javaLangClassClass =
@@ -220,6 +237,14 @@ implements   ClassVisitor,
     }
 
 
+    public void visitMethodTypeConstant(Clazz clazz, MethodTypeConstant methodTypeConstant)
+    {
+        // Fill out the MethodType class.
+        methodTypeConstant.javaLangInvokeMethodTypeClass =
+            findClass(clazz.getName(), ClassConstants.INTERNAL_NAME_JAVA_LANG_INVOKE_METHOD_TYPE);
+    }
+
+
     // Implementations for AttributeVisitor.
 
     public void visitAnyAttribute(Clazz clazz, Attribute attribute) {}
@@ -231,53 +256,36 @@ implements   ClassVisitor,
         String enclosingClassName = enclosingMethodAttribute.getClassName(clazz);
 
         // See if we can find the referenced class.
-        Clazz referencedClass = findClass(className, enclosingClassName);
+        enclosingMethodAttribute.referencedClass =
+            findClass(className, enclosingClassName);
 
-        if (referencedClass == null)
+        if (enclosingMethodAttribute.referencedClass != null)
         {
-            // We couldn't find the enclosing class.
-            if (missingWarningPrinter != null)
+            // Is there an enclosing method? Otherwise it's just initialization
+            // code outside of the constructors.
+            if (enclosingMethodAttribute.u2nameAndTypeIndex != 0)
             {
-                missingWarningPrinter.print("Warning: " +
-                                     ClassUtil.externalClassName(className) +
-                                     ": can't find enclosing class " +
-                                     ClassUtil.externalClassName(enclosingClassName));
+                String name = enclosingMethodAttribute.getName(clazz);
+                String type = enclosingMethodAttribute.getType(clazz);
+
+                // See if we can find the method in the referenced class.
+                enclosingMethodAttribute.referencedMethod =
+                    enclosingMethodAttribute.referencedClass.findMethod(name, type);
+
+                if (enclosingMethodAttribute.referencedMethod == null)
+                {
+                    // We couldn't find the enclosing method.
+                    missingMemberWarningPrinter.print(className,
+                                                      enclosingClassName,
+                                                      "Warning: " +
+                                                      ClassUtil.externalClassName(className) +
+                                                      ": can't find enclosing method '" +
+                                                      ClassUtil.externalFullMethodDescription(enclosingClassName, 0, name, type) +
+                                                      "' in class " +
+                                                      ClassUtil.externalClassName(enclosingClassName));
+                }
             }
-
-            return;
         }
-
-        // Make sure there is actually an enclosed method.
-        if (enclosingMethodAttribute.u2nameAndTypeIndex == 0)
-        {
-            return;
-        }
-
-        String name = enclosingMethodAttribute.getName(clazz);
-        String type = enclosingMethodAttribute.getType(clazz);
-
-        // See if we can find the method in the referenced class.
-        Method referencedMethod = referencedClass.findMethod(name, type);
-
-        if (referencedMethod == null)
-        {
-            // We couldn't find the enclosing method.
-            if (missingWarningPrinter != null)
-            {
-                missingWarningPrinter.print("Warning: " +
-                                     ClassUtil.externalClassName(className) +
-                                     ": can't find enclosing method '" +
-                                     ClassUtil.externalFullMethodDescription(enclosingClassName, 0, name, type) +
-                                     "' in class " +
-                                     ClassUtil.externalClassName(enclosingClassName));
-            }
-
-            return;
-        }
-
-        // Save the references.
-        enclosingMethodAttribute.referencedClass  = referencedClass;
-        enclosingMethodAttribute.referencedMethod = referencedMethod;
     }
 
 
@@ -501,11 +509,17 @@ implements   ClassVisitor,
      */
     private Clazz findClass(String referencingClassName, String name)
     {
-        // Ignore any primitive array types.
-        if (ClassUtil.isInternalArrayType(name) &&
-            !ClassUtil.isInternalClassType(name))
+        // Is it an array type?
+        if (ClassUtil.isInternalArrayType(name))
         {
-            return null;
+            // Ignore any primitive array types.
+            if (!ClassUtil.isInternalClassType(name))
+            {
+                return null;
+            }
+
+            // Strip the array part.
+            name = ClassUtil.internalClassNameFromClassType(name);
         }
 
         // First look for the class in the program class pool.
@@ -517,20 +531,24 @@ implements   ClassVisitor,
             clazz = libraryClassPool.getClass(name);
 
             if (clazz == null &&
-                missingWarningPrinter != null)
+                missingClassWarningPrinter != null)
             {
                 // We didn't find the superclass or interface. Print a warning.
-                missingWarningPrinter.print("Warning: " +
-                                            ClassUtil.externalClassName(referencingClassName) +
-                                            ": can't find referenced class " +
-                                            ClassUtil.externalClassName(name));
+                missingClassWarningPrinter.print(referencingClassName,
+                                                 name,
+                                                 "Warning: " +
+                                                 ClassUtil.externalClassName(referencingClassName) +
+                                                 ": can't find referenced class " +
+                                                 ClassUtil.externalClassName(name));
             }
         }
         else if (dependencyWarningPrinter != null)
         {
             // The superclass or interface was found in the program class pool.
             // Print a warning.
-            dependencyWarningPrinter.print("Warning: library class " +
+            dependencyWarningPrinter.print(referencingClassName,
+                                           name,
+                                           "Warning: library class " +
                                            ClassUtil.externalClassName(referencingClassName) +
                                            " depends on program class " +
                                            ClassUtil.externalClassName(name));
