@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -62,7 +62,8 @@ implements   ConstantVisitor
         {
             // See if we can find the referenced class.
             stringConstant.referencedClass =
-                findClass(ClassUtil.internalClassName(stringConstant.getString(clazz)));
+                findClass(ClassUtil.internalClassName(
+                          ClassUtil.externalBaseType(stringConstant.getString(clazz))));
         }
     }
 
