@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -23,6 +23,8 @@ package proguard.classfile.visitor;
 import proguard.classfile.*;
 import proguard.util.*;
 
+import java.util.List;
+
 /**
  * This <code>ClassVisitor</code> delegates its visits to another given
  * <code>ClassVisitor</code>, but only when the visited class has a name that
@@ -53,7 +55,22 @@ public class ClassNameFilter implements ClassVisitor
 
     /**
      * Creates a new ClassNameFilter.
-     * @param regularExpressionMatcher the regular expression against which
+     * @param regularExpression the regular expression against which class names
+     *                          will be matched.
+     * @param classVisitor      the <code>ClassVisitor</code> to which visits
+     *                          will be delegated.
+     */
+    public ClassNameFilter(List         regularExpression,
+                           ClassVisitor classVisitor)
+    {
+        this(new ListParser(new ClassNameParser()).parse(regularExpression),
+             classVisitor);
+    }
+
+
+    /**
+     * Creates a new ClassNameFilter.
+     * @param regularExpressionMatcher the string matcher against which
      *                                 class names will be matched.
      * @param classVisitor             the <code>ClassVisitor</code> to which
      *                                 visits will be delegated.
