@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2009 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -48,8 +48,9 @@ implements   ClassVisitor,
              ConstantVisitor,
              MemberVisitor,
              AttributeVisitor,
-             ExceptionInfoVisitor,
+             BootstrapMethodInfoVisitor,
              InnerClassesInfoVisitor,
+             ExceptionInfoVisitor,
              StackMapFrameVisitor,
              VerificationTypeVisitor,
              LineNumberInfoVisitor,
@@ -62,7 +63,8 @@ implements   ClassVisitor,
     private static final String INDENTATION = "  ";
 
     private final PrintStream ps;
-    private int         indentation;
+
+    private int indentation;
 
 
     /**
@@ -222,10 +224,30 @@ implements   ClassVisitor,
     }
 
 
+    public void visitInvokeDynamicConstant(Clazz clazz, InvokeDynamicConstant invokeDynamicConstant)
+    {
+        println(visitorInfo(invokeDynamicConstant) + " InvokeDynamic [bootstrap method index = " + invokeDynamicConstant.u2bootstrapMethodAttributeIndex + "]:");
+
+        indent();
+        clazz.constantPoolEntryAccept(invokeDynamicConstant.u2nameAndTypeIndex, this);
+        outdent();
+    }
+
+
+    public void visitMethodHandleConstant(Clazz clazz, MethodHandleConstant methodHandleConstant)
+    {
+        println(visitorInfo(methodHandleConstant) + " MethodHandle [kind = " + methodHandleConstant.u1referenceKind + "]:");
+
+        indent();
+        clazz.constantPoolEntryAccept(methodHandleConstant.u2referenceIndex, this);
+        outdent();
+    }
+
+
     public void visitFieldrefConstant(Clazz clazz, FieldrefConstant fieldrefConstant)
     {
         println(visitorInfo(fieldrefConstant) + " Fieldref [" +
-                clazz.getClassName(fieldrefConstant.u2classIndex)  + "." +
+                clazz.getClassName(fieldrefConstant.u2classIndex) + "." +
                 clazz.getName(fieldrefConstant.u2nameAndTypeIndex) + " " +
                 clazz.getType(fieldrefConstant.u2nameAndTypeIndex) + "]");
     }
@@ -256,6 +278,13 @@ implements   ClassVisitor,
     }
 
 
+    public void visitMethodTypeConstant(Clazz clazz, MethodTypeConstant methodTypeConstant)
+    {
+        println(visitorInfo(methodTypeConstant) + " MethodType [" +
+                clazz.getString(methodTypeConstant.u2descriptorIndex) + "]");
+    }
+
+
     public void visitNameAndTypeConstant(Clazz clazz, NameAndTypeConstant nameAndTypeConstant)
     {
         println(visitorInfo(nameAndTypeConstant) + " NameAndType [" +
@@ -361,6 +390,17 @@ implements   ClassVisitor,
     }
 
 
+    public void visitBootstrapMethodsAttribute(Clazz clazz, BootstrapMethodsAttribute bootstrapMethodsAttribute)
+    {
+        println(visitorInfo(bootstrapMethodsAttribute) +
+                " Bootstrap methods attribute (count = " + bootstrapMethodsAttribute.u2bootstrapMethodsCount + "):");
+
+        indent();
+        bootstrapMethodsAttribute.bootstrapMethodEntriesAccept(clazz, this);
+        outdent();
+    }
+
+
     public void visitSourceFileAttribute(Clazz clazz, SourceFileAttribute sourceFileAttribute)
     {
         println(visitorInfo(sourceFileAttribute) +
@@ -595,6 +635,21 @@ implements   ClassVisitor,
     }
 
 
+    // Implementations for BootstrapMethodInfoVisitor.
+
+    public void visitBootstrapMethodInfo(Clazz clazz, BootstrapMethodInfo bootstrapMethodInfo)
+    {
+        println(visitorInfo(bootstrapMethodInfo) +
+                " BootstrapMethodInfo (argument count = " +
+                bootstrapMethodInfo.u2methodArgumentCount+ "):");
+
+        indent();
+        clazz.constantPoolEntryAccept(bootstrapMethodInfo.u2methodHandleIndex, this);
+        bootstrapMethodInfo.methodArgumentsAccept(clazz, this);
+        outdent();
+    }
+
+
     // Implementations for InnerClassesInfoVisitor.
 
     public void visitInnerClassesInfo(Clazz clazz, InnerClassesInfo innerClassesInfo)
@@ -603,6 +658,8 @@ implements   ClassVisitor,
                 " InnerClassesInfo:");
 
         indent();
+        println("Access flags:  0x" + Integer.toHexString(innerClassesInfo.u2innerClassAccessFlags) + " = " +
+                ClassUtil.externalClassAccessFlags(innerClassesInfo.u2innerClassAccessFlags));
         innerClassesInfo.innerClassConstantAccept(clazz, this);
         innerClassesInfo.outerClassConstantAccept(clazz, this);
         innerClassesInfo.innerNameConstantAccept(clazz, this);
@@ -816,7 +873,7 @@ implements   ClassVisitor,
 
     public void visitLocalVariableInfo(Clazz clazz, Method method, CodeAttribute codeAttribute, LocalVariableInfo localVariableInfo)
     {
-        println("#" + localVariableInfo.u2index + ": " +
+        println("v" + localVariableInfo.u2index + ": " +
                 localVariableInfo.u2startPC + " -> " +
                 (localVariableInfo.u2startPC + localVariableInfo.u2length) + " [" +
                 clazz.getString(localVariableInfo.u2descriptorIndex) + " " +
@@ -828,7 +885,7 @@ implements   ClassVisitor,
 
     public void visitLocalVariableTypeInfo(Clazz clazz, Method method, CodeAttribute codeAttribute, LocalVariableTypeInfo localVariableTypeInfo)
     {
-        println("#" + localVariableTypeInfo.u2index + ": " +
+        println("v" + localVariableTypeInfo.u2index + ": " +
                 localVariableTypeInfo.u2startPC + " -> " +
                 (localVariableTypeInfo.u2startPC + localVariableTypeInfo.u2length) + " [" +
                 clazz.getString(localVariableTypeInfo.u2signatureIndex) + " " +
