@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -22,11 +22,8 @@ package proguard.evaluation;
 
 import proguard.classfile.*;
 import proguard.classfile.attribute.CodeAttribute;
-import proguard.classfile.constant.*;
-import proguard.classfile.constant.visitor.ConstantVisitor;
 import proguard.classfile.instruction.*;
 import proguard.classfile.instruction.visitor.InstructionVisitor;
-import proguard.classfile.util.SimplifiedVisitor;
 import proguard.evaluation.value.*;
 
 /**
@@ -36,9 +33,7 @@ import proguard.evaluation.value.*;
  * @author Eric Lafortune
  */
 public class Processor
-extends      SimplifiedVisitor
-implements   InstructionVisitor,
-             ConstantVisitor
+implements   InstructionVisitor
 {
     private final Variables      variables;
     private final Stack          stack;
@@ -46,9 +41,8 @@ implements   InstructionVisitor,
     private final BranchUnit     branchUnit;
     private final InvocationUnit invocationUnit;
 
-    // Fields acting as parameters for the ConstantVisitor methods.
-    private boolean handleClassConstantAsClassValue;
-    private Value   cpValue;
+    private final ConstantValueFactory      constantValueFactory;
+    private final ClassConstantValueFactory classConstantValueFactory;
 
 
     /**
@@ -69,6 +63,9 @@ implements   InstructionVisitor,
         this.valueFactory   = valueFactory;
         this.branchUnit     = branchUnit;
         this.invocationUnit = invocationUnit;
+
+        constantValueFactory      = new ConstantValueFactory(valueFactory);
+        classConstantValueFactory = new ClassConstantValueFactory(valueFactory);
     }
 
 
@@ -406,63 +403,63 @@ implements   InstructionVisitor,
                 break;
 
             case InstructionConstants.OP_I2L:
-                stack.push(stack.ipop().convertToLong(valueFactory));
+                stack.push(stack.ipop().convertToLong());
                 break;
 
             case InstructionConstants.OP_I2F:
-                stack.push(stack.ipop().convertToFloat(valueFactory));
+                stack.push(stack.ipop().convertToFloat());
                 break;
 
             case InstructionConstants.OP_I2D:
-                stack.push(stack.ipop().convertToDouble(valueFactory));
+                stack.push(stack.ipop().convertToDouble());
                 break;
 
             case InstructionConstants.OP_L2I:
-                stack.push(stack.lpop().convertToInteger(valueFactory));
+                stack.push(stack.lpop().convertToInteger());
                 break;
 
             case InstructionConstants.OP_L2F:
-                stack.push(stack.lpop().convertToFloat(valueFactory));
+                stack.push(stack.lpop().convertToFloat());
                 break;
 
             case InstructionConstants.OP_L2D:
-                stack.push(stack.lpop().convertToDouble(valueFactory));
+                stack.push(stack.lpop().convertToDouble());
                 break;
 
             case InstructionConstants.OP_F2I:
-                stack.push(stack.fpop().convertToInteger(valueFactory));
+                stack.push(stack.fpop().convertToInteger());
                 break;
 
             case InstructionConstants.OP_F2L:
-                stack.push(stack.fpop().convertToLong(valueFactory));
+                stack.push(stack.fpop().convertToLong());
                 break;
 
             case InstructionConstants.OP_F2D:
-                stack.push(stack.fpop().convertToDouble(valueFactory));
+                stack.push(stack.fpop().convertToDouble());
                 break;
 
             case InstructionConstants.OP_D2I:
-                stack.push(stack.dpop().convertToInteger(valueFactory));
+                stack.push(stack.dpop().convertToInteger());
                 break;
 
             case InstructionConstants.OP_D2L:
-                stack.push(stack.dpop().convertToLong(valueFactory));
+                stack.push(stack.dpop().convertToLong());
                 break;
 
             case InstructionConstants.OP_D2F:
-                stack.push(stack.dpop().convertToFloat(valueFactory));
+                stack.push(stack.dpop().convertToFloat());
                 break;
 
             case InstructionConstants.OP_I2B:
-                stack.push(stack.ipop().convertToByte(valueFactory));
+                stack.push(stack.ipop().convertToByte());
                 break;
 
             case InstructionConstants.OP_I2C:
-                stack.push(stack.ipop().convertToCharacter(valueFactory));
+                stack.push(stack.ipop().convertToCharacter());
                 break;
 
             case InstructionConstants.OP_I2S:
-                stack.push(stack.ipop().convertToShort(valueFactory));
+                stack.push(stack.ipop().convertToShort());
                 break;
 
             case InstructionConstants.OP_LCMP:
@@ -470,27 +467,27 @@ implements   InstructionVisitor,
 
                 LongValue longValue1 = stack.lpop();
                 LongValue longValue2 = stack.lpop();
-                stack.push(longValue2.compare(longValue1, valueFactory));
+                stack.push(longValue2.compare(longValue1));
                 break;
 
             case InstructionConstants.OP_FCMPL:
                 FloatValue floatValue1 = stack.fpop();
                 FloatValue floatValue2 = stack.fpop();
-                stack.push(floatValue2.compare(floatValue1, valueFactory));
+                stack.push(floatValue2.compare(floatValue1));
                 break;
 
             case InstructionConstants.OP_FCMPG:
-                stack.push(stack.fpop().compareReverse(stack.fpop(), valueFactory));
+                stack.push(stack.fpop().compareReverse(stack.fpop()));
                 break;
 
             case InstructionConstants.OP_DCMPL:
                 DoubleValue doubleValue1 = stack.dpop();
                 DoubleValue doubleValue2 = stack.dpop();
-                stack.push(doubleValue2.compare(doubleValue1, valueFactory));
+                stack.push(doubleValue2.compare(doubleValue1));
                 break;
 
             case InstructionConstants.OP_DCMPG:
-                stack.push(stack.dpop().compareReverse(stack.dpop(), valueFactory));
+                stack.push(stack.dpop().compareReverse(stack.dpop()));
                 break;
 
             case InstructionConstants.OP_IRETURN:
@@ -561,7 +558,7 @@ implements   InstructionVisitor,
             case InstructionConstants.OP_LDC:
             case InstructionConstants.OP_LDC_W:
             case InstructionConstants.OP_LDC2_W:
-                stack.push(cpValue(clazz, constantIndex, true));
+                stack.push(classConstantValueFactory.constantValue(clazz, constantIndex));
                 break;
 
             case InstructionConstants.OP_GETSTATIC:
@@ -572,16 +569,17 @@ implements   InstructionVisitor,
             case InstructionConstants.OP_INVOKESPECIAL:
             case InstructionConstants.OP_INVOKESTATIC:
             case InstructionConstants.OP_INVOKEINTERFACE:
+            case InstructionConstants.OP_INVOKEDYNAMIC:
                 invocationUnit.invokeMember(clazz, method, codeAttribute, offset, constantInstruction, stack);
                 break;
 
             case InstructionConstants.OP_NEW:
-                stack.push(cpValue(clazz, constantIndex).referenceValue());
+                stack.push(constantValueFactory.constantValue(clazz, constantIndex).referenceValue());
                 break;
 
             case InstructionConstants.OP_ANEWARRAY:
             {
-                ReferenceValue referenceValue = cpValue(clazz, constantIndex).referenceValue();
+                ReferenceValue referenceValue = constantValueFactory.constantValue(clazz, constantIndex).referenceValue();
 
                 stack.push(valueFactory.createArrayReferenceValue(referenceValue.internalType(),
                                                                   referenceValue.getReferencedClass(),
@@ -594,14 +592,14 @@ implements   InstructionVisitor,
                 ReferenceValue castValue = stack.apop();
                 ReferenceValue castResultValue =
                     castValue.isNull() == Value.ALWAYS ? castValue :
-                    castValue.isNull() == Value.NEVER  ? cpValue(clazz, constantIndex).referenceValue() :
-                                                         cpValue(clazz, constantIndex).referenceValue().generalize(valueFactory.createReferenceValueNull());
+                    castValue.isNull() == Value.NEVER  ? constantValueFactory.constantValue(clazz, constantIndex).referenceValue() :
+                                                         constantValueFactory.constantValue(clazz, constantIndex).referenceValue().generalize(valueFactory.createReferenceValueNull());
                 stack.push(castResultValue);
                 break;
 
             case InstructionConstants.OP_INSTANCEOF:
             {
-                ReferenceValue referenceValue = cpValue(clazz, constantIndex).referenceValue();
+                ReferenceValue referenceValue = constantValueFactory.constantValue(clazz, constantIndex).referenceValue();
 
                 int instanceOf = stack.apop().instanceOf(referenceValue.getType(),
                                                          referenceValue.getReferencedClass());
@@ -621,7 +619,7 @@ implements   InstructionVisitor,
                     IntegerValue arrayLength = stack.ipop();
                 }
 
-                stack.push(cpValue(clazz, constantIndex).referenceValue());
+                stack.push(constantValueFactory.constantValue(clazz, constantIndex).referenceValue());
                 break;
             }
 
@@ -907,73 +905,4 @@ implements   InstructionVisitor,
             }
         }
     }
-
-
-    // Implementations for ConstantVisitor.
-
-    public void visitIntegerConstant(Clazz clazz, IntegerConstant integerConstant)
-    {
-        cpValue = valueFactory.createIntegerValue(integerConstant.getValue());
-    }
-
-    public void visitLongConstant(Clazz clazz, LongConstant longConstant)
-    {
-        cpValue = valueFactory.createLongValue(longConstant.getValue());
-    }
-
-    public void visitFloatConstant(Clazz clazz, FloatConstant floatConstant)
-    {
-        cpValue = valueFactory.createFloatValue(floatConstant.getValue());
-    }
-
-    public void visitDoubleConstant(Clazz clazz, DoubleConstant doubleConstant)
-    {
-        cpValue = valueFactory.createDoubleValue(doubleConstant.getValue());
-    }
-
-    public void visitStringConstant(Clazz clazz, StringConstant stringConstant)
-    {
-        cpValue = valueFactory.createReferenceValue(ClassConstants.INTERNAL_NAME_JAVA_LANG_STRING,
-                                                    stringConstant.javaLangStringClass, 
-                                                    false);
-    }
-
-    public void visitClassConstant(Clazz clazz, ClassConstant classConstant)
-    {
-        cpValue = handleClassConstantAsClassValue ?
-            valueFactory.createReferenceValue(ClassConstants.INTERNAL_NAME_JAVA_LANG_CLASS,
-                                              classConstant.javaLangClassClass,
-                                              false) :
-            valueFactory.createReferenceValue(classConstant.getName(clazz),
-                                              classConstant.referencedClass,
-                                              false);
-    }
-
-
-    // Small utility methods.
-
-    /**
-     * Returns the Value of the constant pool element at the given index.
-     */
-    private Value cpValue(Clazz clazz,
-                          int   constantIndex)
-    {
-        return cpValue(clazz, constantIndex, false);
-    }
-
-
-    /**
-     * Returns the Value of the constant pool element at the given index.
-     */
-    private Value cpValue(Clazz   clazz,
-                          int     constantIndex,
-                          boolean handleClassConstantAsClassValue)
-    {
-        this.handleClassConstantAsClassValue = handleClassConstantAsClassValue;
-
-        // Visit the constant pool entry to get its return value.
-        clazz.constantPoolEntryAccept(constantIndex, this);
-
-        return cpValue;
-    }
 }
