@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -153,9 +153,12 @@ abstract class SpecificDoubleValue extends DoubleValue
 
     public IntegerValue compare(SpecificDoubleValue other)
     {
-        return this.equals(other) ?
-            SpecificValueFactory.INTEGER_VALUE_0 :
-            new ComparisonValue(this, other);
+        return ValueFactory.INTEGER_VALUE;
+
+        // Not handling NaN properly.
+        //return this.equals(other) ?
+        //    SpecificValueFactory.INTEGER_VALUE_0 :
+        //    new ComparisonValue(this, other);
     }
 
 
