@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -56,6 +56,7 @@ final class MemberSpecificationDialog extends JDialog
     private final JRadioButton[] protectedRadioButtons;
     private final JRadioButton[] staticRadioButtons;
     private final JRadioButton[] finalRadioButtons;
+    private final JRadioButton[] syntheticRadioButtons;
 
     private JRadioButton[] volatileRadioButtons;
     private JRadioButton[] transientRadioButtons;
@@ -64,6 +65,8 @@ final class MemberSpecificationDialog extends JDialog
     private JRadioButton[] nativeRadioButtons;
     private JRadioButton[] abstractRadioButtons;
     private JRadioButton[] strictRadioButtons;
+    private JRadioButton[] bridgeRadioButtons;
+    private JRadioButton[] varargsRadioButtons;
 
     private final JTextField annotationTypeTextField = new JTextField(20);
     private final JTextField nameTextField           = new JTextField(20);
@@ -166,6 +169,7 @@ final class MemberSpecificationDialog extends JDialog
         protectedRadioButtons = addRadioButtonTriplet("Protected", accessPanel);
         staticRadioButtons    = addRadioButtonTriplet("Static",    accessPanel);
         finalRadioButtons     = addRadioButtonTriplet("Final",     accessPanel);
+        syntheticRadioButtons = addRadioButtonTriplet("Synthetic", accessPanel);
 
         if (isField)
         {
@@ -178,6 +182,8 @@ final class MemberSpecificationDialog extends JDialog
             nativeRadioButtons       = addRadioButtonTriplet("Native",       accessPanel);
             abstractRadioButtons     = addRadioButtonTriplet("Abstract",     accessPanel);
             strictRadioButtons       = addRadioButtonTriplet("Strict",       accessPanel);
+            bridgeRadioButtons       = addRadioButtonTriplet("Bridge",       accessPanel);
+            varargsRadioButtons      = addRadioButtonTriplet("Varargs",      accessPanel);
         }
 
         // Create the type panel.
@@ -332,12 +338,15 @@ final class MemberSpecificationDialog extends JDialog
         setMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_PROTECTED,    protectedRadioButtons);
         setMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_STATIC,       staticRadioButtons);
         setMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_FINAL,        finalRadioButtons);
+        setMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_SYNTHETIC,    syntheticRadioButtons);
         setMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_VOLATILE,     volatileRadioButtons);
         setMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_TRANSIENT,    transientRadioButtons);
         setMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_SYNCHRONIZED, synchronizedRadioButtons);
         setMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_NATIVE,       nativeRadioButtons);
         setMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_ABSTRACT,     abstractRadioButtons);
         setMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_STRICT,       strictRadioButtons);
+        setMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_BRIDGE,       bridgeRadioButtons);
+        setMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_VARARGS,      varargsRadioButtons);
 
         // Set the class name text fields.
         nameTextField.setText(name == null ? "*" : name);
@@ -403,12 +412,15 @@ final class MemberSpecificationDialog extends JDialog
         getMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_PROTECTED,    protectedRadioButtons);
         getMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_STATIC,       staticRadioButtons);
         getMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_FINAL,        finalRadioButtons);
+        getMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_SYNTHETIC,    syntheticRadioButtons);
         getMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_VOLATILE,     volatileRadioButtons);
         getMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_TRANSIENT,    transientRadioButtons);
         getMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_SYNCHRONIZED, synchronizedRadioButtons);
         getMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_NATIVE,       nativeRadioButtons);
         getMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_ABSTRACT,     abstractRadioButtons);
         getMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_STRICT,       strictRadioButtons);
+        getMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_BRIDGE,       bridgeRadioButtons);
+        getMemberSpecificationRadioButtons(memberSpecification, ClassConstants.INTERNAL_ACC_VARARGS,      varargsRadioButtons);
 
         return memberSpecification;
     }
