@@ -2,21 +2,21 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2007 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
- * This library is free software; you can redistribute it and/or modify it
+ * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
  * Software Foundation; either version 2 of the License, or (at your option)
  * any later version.
  *
- * This library is distributed in the hope that it will be useful, but WITHOUT
+ * This program is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
- * for more details.
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
+ * more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 package proguard.optimize.evaluation;
 
@@ -78,6 +78,16 @@ implements   AttributeVisitor,
     }
 
 
+    /**
+     * Returns whether the instruction at the given offset has ever been
+     * executed during the partial evaluation.
+     */
+    public boolean isTraced(int instructionOffset)
+    {
+        return partialEvaluator.isTraced(instructionOffset);
+    }
+
+
     /**
      * Returns whether the specified variable is alive before the instruction
      * at the given offset.
@@ -348,7 +358,7 @@ implements   AttributeVisitor,
                 // Start marking the variable before the load instruction.
                 alive |= livenessMask;
             }
-            else if (variableInstruction.opcode != InstructionConstants.OP_IINC)
+            else
             {
                 // Stop marking the variable before the store instruction.
                 alive &= ~livenessMask;
@@ -429,6 +439,10 @@ implements   AttributeVisitor,
     }
 
 
+    /**
+     * Returns the combined liveness mask of the variables right before the
+     * specified instruction offsets.
+     */
     private long combinedLiveness(InstructionOffsetValue instructionOffsetValue)
     {
         long alive = 0L;
