@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2008 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -33,6 +33,8 @@ public class ClassOptimizationInfo
     private boolean isInstantiated                = false;
     private boolean isInstanceofed                = false;
     private boolean isDotClassed                  = false;
+    private boolean isCaught                      = false;
+    private boolean containsStaticInitializer     = false;
     private boolean containsPackageVisibleMembers = false;
     private boolean invokesPackageVisibleMembers  = false;
     private Clazz   targetClass;
@@ -74,6 +76,30 @@ public class ClassOptimizationInfo
     }
 
 
+    public void setCaught()
+    {
+        isCaught = true;
+    }
+
+
+    public boolean isCaught()
+    {
+        return isCaught;
+    }
+
+
+    public void setContainsStaticInitializer()
+    {
+        containsStaticInitializer = true;
+    }
+
+
+    public boolean containsStaticInitializer()
+    {
+        return containsStaticInitializer;
+    }
+
+
     public void setContainsPackageVisibleMembers()
     {
         containsPackageVisibleMembers = true;
@@ -110,6 +136,18 @@ public class ClassOptimizationInfo
     }
 
 
+    public void merge(ClassOptimizationInfo other)
+    {
+        this.isInstantiated                |= other.isInstantiated;
+        this.isInstanceofed                |= other.isInstanceofed;
+        this.isDotClassed                  |= other.isDotClassed;
+        this.isCaught                      |= other.isCaught;
+        this.containsStaticInitializer     |= other.containsStaticInitializer;
+        this.containsPackageVisibleMembers |= other.containsPackageVisibleMembers;
+        this.invokesPackageVisibleMembers  |= other.invokesPackageVisibleMembers;
+    }
+
+
     public static void setClassOptimizationInfo(Clazz clazz)
     {
         clazz.setVisitorInfo(new ClassOptimizationInfo());
