@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -28,6 +28,8 @@ import proguard.classfile.constant.visitor.ConstantVisitor;
 import proguard.classfile.util.SimplifiedVisitor;
 import proguard.classfile.visitor.ClassVisitor;
 
+import java.util.Arrays;
+
 /**
  * This ClassVisitor removes InnerClasses and EnclosingMethod attributes in
  * classes that are retargeted or that refer to classes that are retargeted.
@@ -70,12 +72,9 @@ implements   ClassVisitor,
         }
 
         // Clean up any remaining array elements.
-        for (int index = newAtributesCount; index < attributesCount; index++)
-        {
-            attributes[index] = null;
-        }
+        Arrays.fill(attributes, newAtributesCount, attributesCount, null);
 
-        // Update the number of attribuets.
+        // Update the number of attributes.
         programClass.u2attributesCount = newAtributesCount;
     }
 
@@ -115,10 +114,7 @@ implements   ClassVisitor,
             }
 
             // Clean up any remaining array elements.
-            for (int index = newClassesCount; index < classesCount; index++)
-            {
-                classes[index] = null;
-            }
+            Arrays.fill(classes, newClassesCount, classesCount, null);
 
             // Update the number of classes.
             innerClassesAttribute.u2classesCount = newClassesCount;
