@@ -2,7 +2,7 @@
  * ProGuard -- shrinking, optimization, obfuscation, and preverification
  *             of Java bytecode.
  *
- * Copyright (c) 2002-2010 Eric Lafortune (eric@graphics.cornell.edu)
+ * Copyright (c) 2002-2012 Eric Lafortune (eric@graphics.cornell.edu)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -160,7 +160,7 @@ implements   AttributeVisitor,
         {
             // Check the referenced classes.
             classUsed = true;
-            enumConstantElementValue.referencedClassesAccept(usageMarker);
+            enumConstantElementValue.referencedClassesAccept(this);
 
             if (classUsed)
             {
@@ -181,7 +181,7 @@ implements   AttributeVisitor,
         {
             // Check the referenced classes.
             classUsed = true;
-            classElementValue.referencedClassesAccept(usageMarker);
+            classElementValue.referencedClassesAccept(this);
 
             if (classUsed)
             {
